#ifndef _DYNAMIC_API_H_
#define _DYNAMIC_API_H_

#include <windows.h>
#include <shlwapi.h>
#include <psapi.h>
#include <winternl.h>


class DynamicApi
{
public:
	typedef enum _DLL_DIR {
		SYSTEM32,
		EXE,
		CURRENT,
		SPECIFY,
		SEARCH,
		AUTO3264
	} DLL_DIR;

	DynamicApi( const TCHAR *name, BOOL notLoad = FALSE, DLL_DIR dir = SYSTEM32, const TCHAR *path = NULL );
	virtual ~DynamicApi(void);
	BOOL Init();
	const TCHAR *GetPath(void) { return dllPath; };

protected:
	HMODULE dllHandle;
	BOOL notFreeLib;
	TCHAR dllPath[_MAX_PATH];

	FARPROC GetFuncAddress( const char *func );
};



class DeltaEndApi : public DynamicApi
{
public:
	DeltaEndApi(void)
		: DynamicApi( NULL, FALSE, AUTO3264 )
		, pSetHook( NULL )
		, pReleaseHook( NULL )
		, hooked( FALSE )
	{};
	virtual ~DeltaEndApi(void) {};
	BOOL SetHook( HWND );
	BOOL ReleaseHook( void );
	BOOL IsHooked( void ) { return hooked; };

private:
	typedef BOOL (*DLLSetHook)( HWND hWnd );
	typedef BOOL (*DLLReleaseHook)(void);
	DLLSetHook pSetHook;
	DLLReleaseHook pReleaseHook;

	BOOL hooked;
};



class Kernel32Api : public DynamicApi
{
public:
	Kernel32Api(void)
		: DynamicApi( _T("kernel32.dll"), TRUE )
		, pIsWow64Process( NULL )
		, pGetNativeSystemInfo( NULL )
	{};
	virtual ~Kernel32Api(void) {};
	BOOL IsWow64Process( HANDLE, PBOOL );
	BOOL GetNativeSystemInfo( LPSYSTEM_INFO );

private:
	typedef BOOL ( WINAPI* DLLIsWow64Process )( HANDLE hProcess, PBOOL Wow64Process );
	typedef void ( WINAPI* DLLGetNativeSystemInfo )( LPSYSTEM_INFO lpSystemInfo );
	DLLIsWow64Process pIsWow64Process;
	DLLGetNativeSystemInfo pGetNativeSystemInfo;
};



class PsapiApi : public DynamicApi
{
public:
	PsapiApi(void)
		: DynamicApi( _T("psapi.dll") )
		, pGetModuleFileNameEx( NULL )
		, pEnumProcesses( NULL )
		, pEnumProcessModules( NULL )
		, pGetModuleInformation( NULL )
		, pGetProcessImageFileName( NULL )
	{};
	virtual ~PsapiApi(void) {};
	DWORD GetModuleFileNameEx( HANDLE hProcess, HMODULE hModule, LPTSTR lpFilename, DWORD nSize );
	BOOL EnumProcesses( DWORD* pProcessIds, DWORD cb, DWORD* pBytesReturned );
	BOOL EnumProcessModules( HANDLE hProcess, HMODULE* lphModule, DWORD cb, LPDWORD lpcbNeeded );
	BOOL GetModuleInformation( HANDLE hProcess, HMODULE hModule, LPMODULEINFO lpmodinfo, DWORD cb );
	DWORD GetProcessImageFileName( HANDLE hProcess, LPTSTR lpImageFileName, DWORD nSize );

private:
	typedef DWORD (WINAPI* DLLGetModuleFileNameEx)( HANDLE, HMODULE, LPTSTR, DWORD );
	typedef BOOL (WINAPI *DLLEnumProcesses)( DWORD*, DWORD, DWORD* );
	typedef BOOL (WINAPI *DLLEnumProcessModules)( HANDLE, HMODULE*, DWORD, LPDWORD );
	typedef BOOL (WINAPI *DLLGetModuleInformation)( HANDLE, HMODULE, LPMODULEINFO, DWORD );
	typedef DWORD (WINAPI *DLLGetProcessImageFileName)( HANDLE hProcess, LPTSTR lpImageFileName, DWORD nSize );
	DLLGetModuleFileNameEx pGetModuleFileNameEx;
	DLLEnumProcesses pEnumProcesses;
	DLLEnumProcessModules pEnumProcessModules;
	DLLGetModuleInformation pGetModuleInformation;
	DLLGetProcessImageFileName pGetProcessImageFileName;
};



class User32Api : public DynamicApi
{
public:
	User32Api(void)
		: DynamicApi( _T("user32.dll"), TRUE )
		, pChangeWindowMessageFilter( NULL )
	{};
	virtual ~User32Api(void) {};
	BOOL ChangeWindowMessageFilter( UINT message, DWORD dwFlag );

private:
	typedef BOOL(WINAPI *DLLChangeWindowMessageFilter)( UINT, DWORD );
	DLLChangeWindowMessageFilter pChangeWindowMessageFilter;
};



class Shell32Api : public DynamicApi
{
public:
	Shell32Api(void);
	virtual ~Shell32Api(void) {};
	HRESULT DllGetVersion( DLLVERSIONINFO *pdvi );
	BOOL IsOldVersion(void) { return oldVersion; };

private:
	DLLGETVERSIONPROC pDllGetVersion;
	BOOL oldVersion;
};



class Imm32Api : public DynamicApi
{
public:
	Imm32Api(void)
		: DynamicApi( _T("imm32.dll") )
		, pImmDisableIME( NULL )
	{};
	virtual ~Imm32Api(void) {};
	BOOL ImmDisableIME( DWORD idThread );

private:
	typedef BOOL(WINAPI *DLLImmDisableIME)( DWORD );
	DLLImmDisableIME pImmDisableIME;
};



class NtDllApi : public DynamicApi
{
public:
	NtDllApi(void)
		: DynamicApi( _T("ntdll.dll"), TRUE )
		, pLdrGetProcedureAddress( NULL )
		, pRtlInitAnsiString( NULL )
	{};
	virtual ~NtDllApi(void) {};
	NTSTATUS LdrGetProcedureAddress( HMODULE moduleHandle, PANSI_STRING functionName, WORD ordinalNumber, PVOID functionAddress );
	VOID RtlInitAnsiString( PANSI_STRING DestinationString, const char *SourceString );

private:
	typedef NTSTATUS(WINAPI* DLLLdrGetProcedureAddress)( HMODULE, PANSI_STRING, WORD, PVOID );
	typedef VOID (WINAPI* DLLRtlInitAnsiString)( PANSI_STRING, const char* );
	DLLLdrGetProcedureAddress pLdrGetProcedureAddress;
	DLLRtlInitAnsiString pRtlInitAnsiString;
};



extern DeltaEndApi deltaDll;
extern Kernel32Api kernel32;
extern PsapiApi psapi;
extern User32Api user32;
extern Shell32Api shell32;
extern Imm32Api imm32;

#endif//_DYNAMIC_API_H_
